@@ -3,7 +3,7 @@
  * gedit-dialog-replace.c
  * This file is part of gedit
  *
- * Copyright (C) 2001 Paolo Maggi 
+ * Copyright (C) 2001-2002 Paolo Maggi 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 /*
- * Modified by the gedit Team, 1998-2001. See the AUTHORS file for a 
+ * Modified by the gedit Team, 1998-2002. See the AUTHORS file for a 
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  */
@@ -61,10 +61,9 @@ struct _GeditDialogReplace {
 	GtkWidget *replace_entry;
 	GtkWidget *search_entry_list;
 	GtkWidget *replace_entry_list;
-	GtkWidget *case_sensitive;
+	GtkWidget *match_case_checkbutton;
 	GtkWidget *entire_word_checkbutton;
-	GtkWidget *beginning;
-	GtkWidget *cursor;
+	GtkWidget *wrap_around_checkbutton;
 };
 
 struct _GeditDialogFind {
@@ -73,9 +72,9 @@ struct _GeditDialogFind {
 	GtkWidget *search_entry;
 	GtkWidget *search_entry_list;
 	GtkWidget *case_sensitive;
+	GtkWidget *match_case_checkbutton;
 	GtkWidget *entire_word_checkbutton;
-	GtkWidget *beginning;
-	GtkWidget *cursor;
+	GtkWidget *wrap_around_checkbutton;
 };
 
 static void dialog_destroyed (GtkObject *obj,  void **dialog_pointer);
@@ -91,6 +90,19 @@ static void replace_dlg_replace_all_button_pressed (GeditDialogReplace * dialog)
 static GeditDialogReplace *dialog_replace_get_dialog 	(void);
 static GeditDialogFind    *dialog_find_get_dialog 	(void);
 
+static GQuark was_wrap_around_id = 0;
+static GQuark was_entire_word_id = 0;
+static GQuark was_case_sensitive_id = 0;
+
+GQuark 
+gedit_was_wrap_around_quark (void)
+{
+	if (!was_wrap_around_id)
+		was_wrap_around_id = g_quark_from_static_string ("GeditWasWrapAround");
+
+	return was_wrap_around_id;
+}
+
 static void
 dialog_destroyed (GtkObject *obj,  void **dialog_pointer)
 {
@@ -103,6 +115,28 @@ dialog_destroyed (GtkObject *obj,  void **dialog_pointer)
 	}	
 }
 
+static void 
+text_not_found_dialog (const gchar *text, GtkWindow *parent)
+{
+	GtkWidget *message_dlg;
+
+	g_return_if_fail (text != NULL);
+	
+	message_dlg = gtk_message_dialog_new (
+			parent,
+			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+			GTK_MESSAGE_INFO,
+			GTK_BUTTONS_OK,
+			_("The text \"%s\" was not found."), text);
+		
+	gtk_dialog_set_default_response (GTK_DIALOG (message_dlg), GTK_RESPONSE_OK);
+
+	gtk_window_set_resizable (GTK_WINDOW (message_dlg), FALSE);
+	
+	gtk_dialog_run (GTK_DIALOG (message_dlg));
+  	gtk_widget_destroy (message_dlg);
+}
+
 static void
 dialog_replace_response_handler (GtkDialog *dlg, gint res_id,  GeditDialogReplace *replace_dialog)
 {
@@ -140,6 +174,32 @@ dialog_find_response_handler (GtkDialog *dlg, gint res_id,  GeditDialogFind *fin
 	}
 }
 
+static void 
+replace_search_entry_changed (GtkEditable *editable, GeditDialogReplace *dialog)
+{
+	const gchar *search_string;
+	
+	search_string = gtk_entry_get_text (GTK_ENTRY (dialog->search_entry));		
+	g_return_if_fail (search_string != NULL);
+
+	if (strlen (search_string) <= 0)
+	{
+		gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
+			GEDIT_RESPONSE_FIND, FALSE);
+		gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
+			GEDIT_RESPONSE_REPLACE, FALSE);
+		gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
+			GEDIT_RESPONSE_REPLACE_ALL, FALSE);
+	}
+	else
+	{
+		gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
+			GEDIT_RESPONSE_FIND, TRUE);
+		gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
+			GEDIT_RESPONSE_REPLACE_ALL, TRUE);
+	}
+}
+
 static GeditDialogReplace *
 dialog_replace_get_dialog (void)
 {
@@ -209,21 +269,19 @@ dialog_replace_get_dialog (void)
 	dialog->search_entry_list  = glade_xml_get_widget (gui, "search_for_text_entry_list");
 	dialog->replace_entry_list = glade_xml_get_widget (gui, "replace_with_text_entry_list");
 	replace_with_label         = glade_xml_get_widget (gui, "replace_with_label");
-	dialog->case_sensitive     = glade_xml_get_widget (gui, "case_sensitive");
-	dialog->beginning          = glade_xml_get_widget (gui, "beginning_radio_button");
-	dialog->cursor		   = glade_xml_get_widget (gui, "cursor_radio_button");
+	dialog->match_case_checkbutton = glade_xml_get_widget (gui, "match_case_checkbutton");
+	dialog->wrap_around_checkbutton =  glade_xml_get_widget (gui, "wrap_around_checkbutton");
 	dialog->entire_word_checkbutton = glade_xml_get_widget (gui, "entire_word_checkbutton");
 
-	if (!content			||
-	    !dialog->search_entry 	||
-	    !dialog->replace_entry  	||
-	    !dialog->search_entry_list	||
-	    !dialog->replace_entry_list	||  
-	    !replace_with_label 	||
-	    !dialog->case_sensitive 	||
-	    !dialog->entire_word_checkbutton ||
-	    !dialog->cursor	 	||
-	    !dialog->beginning 	) 
+	if (!content				||
+	    !dialog->search_entry 		||
+	    !dialog->replace_entry  		||
+	    !dialog->search_entry_list		||
+	    !dialog->replace_entry_list		||  
+	    !replace_with_label 		||
+	    !dialog->match_case_checkbutton 	||
+	    !dialog->entire_word_checkbutton 	||
+	    !dialog->wrap_around_checkbutton)
 	{
 		g_print
 		    ("Could not find the required widgets inside replace.glade2.\n");
@@ -240,11 +298,14 @@ dialog_replace_get_dialog (void)
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog->dialog),
 					 GEDIT_RESPONSE_FIND);
 
-	g_signal_connect(G_OBJECT (dialog->dialog), "destroy",
-			 G_CALLBACK (dialog_destroyed), &dialog);
+	g_signal_connect (G_OBJECT (dialog->search_entry_list), "changed",
+			  G_CALLBACK (replace_search_entry_changed), dialog);
+	
+	g_signal_connect (G_OBJECT (dialog->dialog), "destroy",
+			  G_CALLBACK (dialog_destroyed), &dialog);
 
-	g_signal_connect(G_OBJECT (dialog->dialog), "response",
-			 G_CALLBACK (dialog_replace_response_handler), dialog);
+	g_signal_connect (G_OBJECT (dialog->dialog), "response",
+			  G_CALLBACK (dialog_replace_response_handler), dialog);
 
 	g_object_unref (G_OBJECT (gui));
 
@@ -253,6 +314,22 @@ dialog_replace_get_dialog (void)
 	return dialog;
 }
 
+static void 
+find_search_entry_changed (GtkEditable *editable, GeditDialogReplace *dialog)
+{
+	const gchar *search_string;
+	
+	search_string = gtk_entry_get_text (GTK_ENTRY (dialog->search_entry));		
+	g_return_if_fail (search_string != NULL);
+
+	if (strlen (search_string) <= 0)
+		gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
+			GEDIT_RESPONSE_FIND, FALSE);
+	else
+		gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
+			GEDIT_RESPONSE_FIND, TRUE);
+}
+
 static GeditDialogFind *
 dialog_find_get_dialog (void)
 {
@@ -307,23 +384,20 @@ dialog_find_get_dialog (void)
 
 	replace_with_label         = glade_xml_get_widget (gui, "replace_with_label");
 	
-	dialog->case_sensitive     = glade_xml_get_widget (gui, "case_sensitive");
-	dialog->beginning          = glade_xml_get_widget (gui, "beginning_radio_button");
-	dialog->cursor		   = glade_xml_get_widget (gui, "cursor_radio_button");
+	dialog->match_case_checkbutton = glade_xml_get_widget (gui, "match_case_checkbutton");
+	dialog->wrap_around_checkbutton =  glade_xml_get_widget (gui, "wrap_around_checkbutton");
 	dialog->entire_word_checkbutton = glade_xml_get_widget (gui, "entire_word_checkbutton");
 
-
 	table                      = glade_xml_get_widget (gui, "table");
 
-	if (!content			||
-	    !dialog->search_entry 	||
-	    !dialog->search_entry_list 	||
-	    !replace_entry		||
-	    !replace_with_label 	||
-	    !dialog->case_sensitive 	||
-            !dialog->cursor	 	||
-	    !dialog->beginning 		||
-	    !dialog->entire_word_checkbutton ||
+	if (!content				||
+	    !dialog->search_entry 		||
+	    !dialog->search_entry_list 		||
+	    !replace_entry			||
+	    !replace_with_label 		||
+	    !dialog->match_case_checkbutton 	||
+	    !dialog->entire_word_checkbutton 	||
+	    !dialog->wrap_around_checkbutton	||
 	    !table) 
 	{
 		g_print
@@ -342,6 +416,9 @@ dialog_find_get_dialog (void)
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog->dialog),
 					 GEDIT_RESPONSE_FIND);
 
+	g_signal_connect (G_OBJECT (dialog->search_entry_list), "changed",
+			  G_CALLBACK (find_search_entry_changed), dialog);
+
 	g_signal_connect(G_OBJECT (dialog->dialog), "destroy",
 			 G_CALLBACK (dialog_destroyed), &dialog);
 
@@ -362,7 +439,11 @@ gedit_dialog_find (void)
 	GeditMDIChild *active_child;
 	GeditDocument *doc;
 	gchar* last_searched_text;
-
+	gboolean was_wrap_around;
+	gboolean was_entire_word;
+	gboolean was_case_sensitive;
+	gpointer data;
+	
 	gedit_debug (DEBUG_SEARCH, "");
 
 	dialog = dialog_find_get_dialog ();
@@ -376,6 +457,12 @@ gedit_dialog_find (void)
 				      (bonobo_mdi_get_active_window
 				       (BONOBO_MDI (gedit_mdi))));
 
+	if (GTK_WIDGET_VISIBLE (dialog->dialog))
+		return;
+
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
+			GEDIT_RESPONSE_FIND, FALSE);
+
 	active_child = GEDIT_MDI_CHILD (bonobo_mdi_get_active_child (BONOBO_MDI (gedit_mdi)));
 	g_return_if_fail (active_child != NULL);
 
@@ -389,9 +476,41 @@ gedit_dialog_find (void)
 		g_free (last_searched_text);	
 	}
 
-	gtk_widget_grab_focus (dialog->search_entry);
+	if (!was_wrap_around_id)
+		was_wrap_around_id = gedit_was_wrap_around_quark ();
+
+	if (!was_entire_word_id)
+		was_entire_word_id = g_quark_from_static_string ("GeditWasEntireWord");
+
+	if (!was_case_sensitive_id)
+		was_case_sensitive_id = g_quark_from_static_string ("GeditWasCaseSensitive");
+
+	data = g_object_get_qdata (G_OBJECT (doc), was_wrap_around_id);
+	if (data == NULL)
+		was_wrap_around = TRUE;
+	else
+		was_wrap_around = GPOINTER_TO_BOOLEAN (data);
+
+	data = g_object_get_qdata (G_OBJECT (doc), was_entire_word_id);
+	if (data == NULL)
+		was_entire_word = FALSE;
+	else
+		was_entire_word = GPOINTER_TO_BOOLEAN (data);
+
+	data = g_object_get_qdata (G_OBJECT (doc), was_case_sensitive_id);
+	if (data == NULL)
+		was_case_sensitive = FALSE;
+	else
+		was_case_sensitive = GPOINTER_TO_BOOLEAN (data);
 	
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->beginning), TRUE);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->match_case_checkbutton), 
+				      was_case_sensitive);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->entire_word_checkbutton),
+				      was_entire_word);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->wrap_around_checkbutton),
+				      was_wrap_around);
+
+	gtk_widget_grab_focus (dialog->search_entry);
 	
 	if (!GTK_WIDGET_VISIBLE (dialog->dialog))
 		gtk_widget_show (dialog->dialog);
@@ -405,6 +524,10 @@ gedit_dialog_replace (void)
 	GeditDocument *doc;
 	gchar* last_searched_text;
 	gchar* last_replace_text;
+	gboolean was_wrap_around;
+	gboolean was_entire_word;
+	gboolean was_case_sensitive;
+	gpointer data;
 
 	gedit_debug (DEBUG_SEARCH, "");
 
@@ -419,6 +542,16 @@ gedit_dialog_replace (void)
 				      (bonobo_mdi_get_active_window
 				       (BONOBO_MDI (gedit_mdi))));
 
+	if (GTK_WIDGET_VISIBLE (dialog->dialog))
+		return;
+
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
+			GEDIT_RESPONSE_FIND, FALSE);
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
+			GEDIT_RESPONSE_REPLACE, FALSE);
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
+			GEDIT_RESPONSE_REPLACE_ALL, FALSE);
+
 	active_child = GEDIT_MDI_CHILD (bonobo_mdi_get_active_child (BONOBO_MDI (gedit_mdi)));
 	g_return_if_fail (active_child != NULL);
 
@@ -439,12 +572,41 @@ gedit_dialog_replace (void)
 		g_free (last_replace_text);	
 	}
 
-	gtk_widget_grab_focus (dialog->search_entry);
+	if (!was_wrap_around_id)
+		was_wrap_around_id = gedit_was_wrap_around_quark ();
 
-	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
-			GEDIT_RESPONSE_REPLACE, FALSE);
+	if (!was_entire_word_id)
+		was_entire_word_id = g_quark_from_static_string ("GeditWasEntireWord");
+
+	if (!was_case_sensitive_id)
+		was_case_sensitive_id = g_quark_from_static_string ("GeditWasCaseSensitive");
+
+	data = g_object_get_qdata (G_OBJECT (doc), was_wrap_around_id);
+	if (data == NULL)
+		was_wrap_around = TRUE;
+	else
+		was_wrap_around = GPOINTER_TO_BOOLEAN (data);
+
+	data = g_object_get_qdata (G_OBJECT (doc), was_entire_word_id);
+	if (data == NULL)
+		was_entire_word = FALSE;
+	else
+		was_entire_word = GPOINTER_TO_BOOLEAN (data);
+
+	data = g_object_get_qdata (G_OBJECT (doc), was_case_sensitive_id);
+	if (data == NULL)
+		was_case_sensitive = FALSE;
+	else
+		was_case_sensitive = GPOINTER_TO_BOOLEAN (data);
 	
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->beginning), TRUE);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->match_case_checkbutton), 
+				      was_case_sensitive);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->entire_word_checkbutton),
+				      was_entire_word);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->wrap_around_checkbutton),
+				      was_wrap_around);
+
+	gtk_widget_grab_focus (dialog->search_entry);
 	
 	if (!GTK_WIDGET_VISIBLE (dialog->dialog))
 		gtk_widget_show (dialog->dialog);
@@ -457,10 +619,11 @@ find_dlg_find_button_pressed (GeditDialogFind *dialog)
 	GeditView* active_view;
 	GeditDocument *doc;
 	const gchar* search_string = NULL;
-	gboolean from_cursor;
 	gboolean case_sensitive;
 	gboolean entire_word;
-	
+	gboolean wrap_around;
+	gboolean found;
+
 	gedit_debug (DEBUG_SEARCH, "");
 
 	if (bonobo_mdi_get_active_child (BONOBO_MDI (gedit_mdi)) == NULL)
@@ -482,48 +645,26 @@ find_dlg_find_button_pressed (GeditDialogFind *dialog)
 		return;
 
 	gnome_entry_prepend_history (GNOME_ENTRY (dialog->search_entry_list), TRUE, search_string);
-	
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->beginning)))
-		from_cursor = FALSE;
-	else
-		from_cursor = TRUE;
-
-	
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->case_sensitive)))
-		case_sensitive = TRUE;
-	else
-		case_sensitive = FALSE;
-
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->entire_word_checkbutton)))
-		entire_word = TRUE;
-	else
-		entire_word = FALSE;
-
-	if (!gedit_document_find (doc, search_string, from_cursor, case_sensitive, entire_word))
-	{	
-		GtkWidget *message_dlg;
-
-		message_dlg = gtk_message_dialog_new (
-				GTK_WINDOW (dialog->dialog),
-				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-				GTK_MESSAGE_INFO,
-				GTK_BUTTONS_OK,
-				_("The text \"%s\" was not found."), search_string);
 		
-		gtk_dialog_set_default_response (GTK_DIALOG (message_dlg), GTK_RESPONSE_OK);
+	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->match_case_checkbutton));
 
-		gtk_window_set_resizable (GTK_WINDOW (message_dlg), FALSE);
+	entire_word = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->entire_word_checkbutton));
 	
-		gtk_dialog_run (GTK_DIALOG (message_dlg));
-  		gtk_widget_destroy (message_dlg);
+	wrap_around = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->wrap_around_checkbutton));
 
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->beginning), TRUE);		
-	}
-	else
-	{
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->cursor), TRUE);		
+	g_object_set_qdata (G_OBJECT (doc), was_wrap_around_id, GBOOLEAN_TO_POINTER (wrap_around ));
+	g_object_set_qdata (G_OBJECT (doc), was_entire_word_id, GBOOLEAN_TO_POINTER (entire_word));
+	g_object_set_qdata (G_OBJECT (doc), was_case_sensitive_id, GBOOLEAN_TO_POINTER (case_sensitive));
+
+	found = gedit_document_find (doc, search_string, TRUE, case_sensitive, entire_word);
+
+	if (!found && wrap_around)
+		found = gedit_document_find (doc, search_string, FALSE, case_sensitive, entire_word);
+
+	if (found)
 		gedit_view_scroll_to_cursor (active_view);
-	}
+	else
+		text_not_found_dialog (search_string, (GTK_WINDOW (dialog->dialog)));
 }
 
 
@@ -536,10 +677,11 @@ replace_dlg_find_button_pressed (GeditDialogReplace *dialog)
 	GeditView* active_view;
 	GeditDocument *doc;
 	const gchar* search_string = NULL;
-	gboolean from_cursor;
 	gboolean case_sensitive;
 	gboolean entire_word;
-	
+	gboolean wrap_around;
+	gboolean found;
+
 	gedit_debug (DEBUG_SEARCH, "");
 
 	if (bonobo_mdi_get_active_child (BONOBO_MDI (gedit_mdi)) == NULL)
@@ -561,54 +703,29 @@ replace_dlg_find_button_pressed (GeditDialogReplace *dialog)
 		return;
 	
 	gnome_entry_prepend_history (GNOME_ENTRY (dialog->search_entry_list), TRUE, search_string);
-
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->beginning)))
-		from_cursor = FALSE;
-	else
-		from_cursor = TRUE;
-
-	
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->case_sensitive)))
-		case_sensitive = TRUE;
-	else
-		case_sensitive = FALSE;
 		
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->entire_word_checkbutton)))
-		entire_word = TRUE;
-	else
-		entire_word = FALSE;
-	
-	if (!gedit_document_find (doc, search_string, from_cursor, case_sensitive, entire_word))
-	{	
-		GtkWidget *message_dlg;
+	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->match_case_checkbutton));
 
-		message_dlg = gtk_message_dialog_new (
-				GTK_WINDOW (dialog->dialog),
-				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-				GTK_MESSAGE_INFO,
-				GTK_BUTTONS_OK,
-				_("The text \"%s\" was not found."), search_string);
-		
-		gtk_dialog_set_default_response (GTK_DIALOG (message_dlg), GTK_RESPONSE_OK);
+	entire_word = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->entire_word_checkbutton));
 
-		gtk_window_set_resizable (GTK_WINDOW (message_dlg), FALSE);
+	wrap_around = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->wrap_around_checkbutton));
 
-		gtk_dialog_run (GTK_DIALOG (message_dlg));
-  		gtk_widget_destroy (message_dlg);
+	g_object_set_qdata (G_OBJECT (doc), was_wrap_around_id, GBOOLEAN_TO_POINTER (wrap_around));
+	g_object_set_qdata (G_OBJECT (doc), was_entire_word_id, GBOOLEAN_TO_POINTER (entire_word));
+	g_object_set_qdata (G_OBJECT (doc), was_case_sensitive_id, GBOOLEAN_TO_POINTER (case_sensitive));
 
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->beginning), TRUE);	
-		gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
-							GEDIT_RESPONSE_REPLACE, FALSE);
-	
-	}
-	else
-	{
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->cursor), TRUE);
-		gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
-							GEDIT_RESPONSE_REPLACE, TRUE);
-		
+	found = gedit_document_find (doc, search_string, TRUE, case_sensitive, entire_word);
+
+	if (!found && wrap_around)
+		found = gedit_document_find (doc, search_string, FALSE, case_sensitive, entire_word);
+
+	if (found)
 		gedit_view_scroll_to_cursor (active_view);
-	}
+	else
+		text_not_found_dialog (search_string, (GTK_WINDOW (dialog->dialog)));
+
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
+							GEDIT_RESPONSE_REPLACE, found);
 }
 
 static void
@@ -624,6 +741,8 @@ replace_dlg_replace_button_pressed (GeditDialogReplace *dialog)
 	gboolean case_sensitive;
 	gboolean entire_word;
 	gint start, end;
+	gboolean wrap_around;
+	gboolean found;
 
 	gedit_debug (DEBUG_SEARCH, "");
 
@@ -667,15 +786,11 @@ replace_dlg_replace_button_pressed (GeditDialogReplace *dialog)
 	gedit_debug (DEBUG_SEARCH, "Sel text: %s", selected_text ? selected_text : "NULL");
 	gedit_debug (DEBUG_SEARCH, "Search string: %s", search_string ? search_string : "NULL");
 
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->case_sensitive)))
-		case_sensitive = TRUE;
-	else
-		case_sensitive = FALSE;
+	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->match_case_checkbutton));
 
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->entire_word_checkbutton)))
-		entire_word = TRUE;
-	else
-		entire_word = FALSE;
+	entire_word = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->entire_word_checkbutton));
+
+	wrap_around = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->wrap_around_checkbutton));
 
 	converted_search_string = gedit_utils_convert_search_text (search_string);
 
@@ -710,21 +825,16 @@ replace_dlg_replace_button_pressed (GeditDialogReplace *dialog)
 	gedit_debug (DEBUG_SEARCH, "Replaced");
 
 	/* go ahead and find the next one */
-	if (!gedit_document_find (doc, search_string, TRUE, case_sensitive, entire_word))
-	{	
-	
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->beginning), TRUE);	
-		gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
-							GEDIT_RESPONSE_REPLACE, FALSE);	
-	}
-	else
-	{
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->cursor), TRUE);
-		gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
-							GEDIT_RESPONSE_REPLACE, TRUE);
-		
+	found = gedit_document_find (doc, search_string, TRUE, case_sensitive, entire_word);
+
+	if (!found && wrap_around)
+		found = gedit_document_find (doc, search_string, FALSE, case_sensitive, entire_word);
+
+	if (found)
 		gedit_view_scroll_to_cursor (active_view);
-	}
+	
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
+							GEDIT_RESPONSE_REPLACE, found);	
 
 	gedit_debug (DEBUG_SEARCH, "END");
 }
@@ -773,17 +883,12 @@ replace_dlg_replace_all_button_pressed (GeditDialogReplace *dialog)
 					     TRUE, 
 					     replace_string);
 
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->case_sensitive)))
-		case_sensitive = TRUE;
-	else
-		case_sensitive = FALSE;
+	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->match_case_checkbutton));
 
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->entire_word_checkbutton)))
-		entire_word = TRUE;
-	else
-		entire_word = FALSE;
+	entire_word = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->entire_word_checkbutton));
 
-	replaced_items = gedit_document_replace_all (doc, search_string, replace_string, case_sensitive, entire_word);
+	replaced_items = gedit_document_replace_all (doc, search_string, replace_string, 
+						     case_sensitive, entire_word);
 		
 	if (replaced_items <= 0)
 	{
