@@ -58,6 +58,8 @@ struct _GeditViewPrivate
 	GtkWidget *overwrite_mode_statusbar;
 
 	gboolean overwrite_mode;
+
+	gint tab_size;
 };
 
 
@@ -98,8 +100,8 @@ static void gedit_view_doc_readonly_changed_handler (GeditDocument *document,
 static gint gedit_view_calculate_real_tab_width (GeditView *view, gint tab_size);
 
 static void gedit_view_populate_popup (GtkTextView *textview, GtkMenu *menu, gpointer data);
-static void gedit_view_undo_activate_callback (GtkWidget *menu_item, GeditDocument *doc);
-static void gedit_view_redo_activate_callback (GtkWidget *menu_item, GeditDocument *doc);
+static void gedit_view_undo_activate_callback (GtkWidget *menu_item, GeditView *view);
+static void gedit_view_redo_activate_callback (GtkWidget *menu_item, GeditView *view);
 
 static void gedit_view_dnd_drop (GtkTextView *view, 
 			     GdkDragContext *context,
@@ -525,6 +527,18 @@ gedit_view_finalize (GObject *object)
 	gedit_debug (DEBUG_VIEW, "END");
 }
 
+static void
+gedit_view_realize_cb (GtkWidget *widget, GeditView *view)
+{
+	gedit_debug (DEBUG_VIEW, "");
+			
+	g_return_if_fail (GEDIT_IS_VIEW (view));
+			
+	/* Set tab size: this function must be called after the widget is
+	 * realized */
+	gedit_view_set_tab_size (view, 
+				 gedit_prefs_manager_get_tabs_size ());
+}
 
 /**
  * gedit_view_new:
@@ -565,10 +579,6 @@ gedit_view_new (GeditDocument *doc)
 			
 	gtk_widget_show_all (GTK_WIDGET (view));
 
-	/* Set tab size: this function must be called after show */
-	gedit_view_set_tab_size (view, gedit_prefs_manager_get_tabs_size ());
-
-
 	tl = gtk_drag_dest_get_target_list (GTK_WIDGET (view->priv->text_view));
 	g_return_val_if_fail (tl != NULL, view);
 
@@ -604,9 +614,13 @@ gedit_view_new (GeditDocument *doc)
 			  G_CALLBACK (gedit_view_key_press_cb),
 			  view);
 
+	g_signal_connect (G_OBJECT (view->priv->text_view),
+			  "realize",
+			  G_CALLBACK (gedit_view_realize_cb),
+			  view);
 
-	gtk_text_view_set_editable (view->priv->text_view, !gedit_document_is_readonly (doc));	
-
+	gtk_text_view_set_editable (view->priv->text_view, 
+				    !gedit_document_is_readonly (doc));	
 	
 	gedit_debug (DEBUG_VIEW, "END: %d", G_OBJECT (view)->ref_count);
 
@@ -866,6 +880,9 @@ gedit_view_calculate_real_tab_width (GeditView *view, gint tab_size)
 	return tab_width;
 }
 
+/* This function must be called after the widget is
+ * realized 
+ */
 void
 gedit_view_set_tab_size (GeditView* view, gint tab_size)
 {
@@ -876,6 +893,9 @@ gedit_view_set_tab_size (GeditView* view, gint tab_size)
 
 	g_return_if_fail (GEDIT_IS_VIEW (view));
 
+	if (view->priv->tab_size == tab_size)
+		return;
+
 	real_tab_width = gedit_view_calculate_real_tab_width (
 					 GEDIT_VIEW (view),
 					 tab_size);
@@ -889,8 +909,15 @@ gedit_view_set_tab_size (GeditView* view, gint tab_size)
 	tab_array = pango_tab_array_new (1, TRUE);
 	pango_tab_array_set_tab (tab_array, 0, PANGO_TAB_LEFT, real_tab_width);
 
-	gtk_text_view_set_tabs (GTK_TEXT_VIEW (view->priv->text_view), tab_array);
+	gtk_text_view_set_tabs (GTK_TEXT_VIEW (view->priv->text_view), 
+				tab_array);
+
 	pango_tab_array_free (tab_array);
+
+	view->priv->tab_size = tab_size;
+	gedit_view_update_cursor_position_statusbar (
+			GTK_TEXT_BUFFER (gedit_view_get_document (view)),
+			view);	
 }
 
 void
@@ -966,14 +993,15 @@ gedit_view_set_overwrite_mode_statusbar (GeditView *view, GtkWidget* status)
 	if (status != NULL)
 		gedit_view_update_overwrite_mode_statusbar (view->priv->text_view, view);
 }
-
+			
 static void
 gedit_view_update_cursor_position_statusbar (GtkTextBuffer *buffer, GeditView* view)
 {
 	gchar *msg;
-	gint row, col;
+	gint row, col/*, chars*/;
 	GtkTextIter iter;
-
+	GtkTextIter start;
+	
 	gedit_debug (DEBUG_VIEW, "");
   
 	if (view->priv->cursor_position_statusbar == NULL)
@@ -987,15 +1015,42 @@ gedit_view_update_cursor_position_statusbar (GtkTextBuffer *buffer, GeditView* v
 					  gtk_text_buffer_get_insert (buffer));
 	
 	row = gtk_text_iter_get_line (&iter);
-	col = gtk_text_iter_get_line_offset (&iter);
-	msg = g_strdup_printf (_("  Ln %d, Col. %d"), row + 1, col + 1);
+	
+	/*
+	chars = gtk_text_iter_get_line_offset (&iter);
+	*/
+	
+	start = iter;
+	gtk_text_iter_set_line_offset (&start, 0);
+	col = 0;
 
+	while (!gtk_text_iter_equal (&start, &iter))
+	{
+		if (gtk_text_iter_get_char (&start) == '\t')
+		{
+			col += (view->priv->tab_size - (col  % view->priv->tab_size));
+		}
+		else
+			++col;
+
+		gtk_text_iter_forward_char (&start);
+	}
+	
+	/*
+	if (col == chars)
+		msg = g_strdup_printf (_("  Ln %d, Col. %d"), row + 1, col + 1);
+	else
+		msg = g_strdup_printf (_("  Ln %d, Col. %d-%d"), row + 1, chars + 1, col + 1);
+	*/
+
+	msg = g_strdup_printf (_("  Ln %d, Col. %d"), row + 1, col + 1);
+	
 	gtk_statusbar_push (GTK_STATUSBAR (view->priv->cursor_position_statusbar), 
 			    0, msg);
 
       	g_free (msg);
 }
-
+	
 static void
 gedit_view_cursor_moved (GtkTextBuffer     *buffer,
 			 const GtkTextIter *new_location,
@@ -1037,23 +1092,47 @@ gedit_view_update_overwrite_mode_statusbar (GtkTextView* w, GeditView* view)
 }
 
 static void
-gedit_view_undo_activate_callback (GtkWidget *menu_item, GeditDocument *doc)
+gedit_view_undo_activate_callback (GtkWidget *menu_item, GeditView *view)
 {
+	GeditDocument *doc;
+	
+	g_return_if_fail (view != NULL);
+	g_return_if_fail (GEDIT_IS_VIEW (view));
+
+	doc = gedit_view_get_document (view);
+
 	g_return_if_fail (doc != NULL);
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 
 	if (gedit_document_can_undo (doc))
+	{
 		gedit_document_undo (doc);
+		gedit_view_scroll_to_cursor (view);
+	}
+
+	gtk_widget_grab_focus (GTK_WIDGET (view));
 }
 
 static void
-gedit_view_redo_activate_callback (GtkWidget *menu_item, GeditDocument *doc)
+gedit_view_redo_activate_callback (GtkWidget *menu_item, GeditView *view)
 {
+	GeditDocument *doc;
+
+	g_return_if_fail (view != NULL);
+	g_return_if_fail (GEDIT_IS_VIEW (view));
+
+	doc = gedit_view_get_document (view);
+	
 	g_return_if_fail (doc != NULL);
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 
 	if (gedit_document_can_redo (doc))
+	{
 		gedit_document_redo (doc);
+		gedit_view_scroll_to_cursor (view);
+	}
+
+	gtk_widget_grab_focus (GTK_WIDGET (view));
 }
 
 static void
@@ -1069,7 +1148,7 @@ gedit_view_populate_popup (GtkTextView *textview, GtkMenu *menu, gpointer data)
 	g_return_if_fail (GEDIT_IS_VIEW (data));
 
 	view = GEDIT_VIEW (data);
-
+	
 	doc = gedit_view_get_document (view);
 	g_return_if_fail (doc != NULL);
 
@@ -1083,14 +1162,14 @@ gedit_view_populate_popup (GtkTextView *textview, GtkMenu *menu, gpointer data)
 	gtk_widget_set_sensitive (menu_item, gedit_document_can_redo (doc));
 	gtk_widget_show (menu_item);
 	g_signal_connect (G_OBJECT (menu_item), "activate",
-		      	  G_CALLBACK (gedit_view_redo_activate_callback), doc);
+		      	  G_CALLBACK (gedit_view_redo_activate_callback), view);
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
 
 	/* Add the undo button */
 	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_UNDO, NULL);
 	gtk_widget_set_sensitive (menu_item, gedit_document_can_undo (doc));
 	g_signal_connect (G_OBJECT (menu_item), "activate",
-		      	  G_CALLBACK (gedit_view_undo_activate_callback), doc);
+		      	  G_CALLBACK (gedit_view_undo_activate_callback), view);
 	gtk_widget_show (menu_item);
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
 
